
-- Snake made by Introversion Software (found hidden inside the game files in autumn 2018)
-- Pong taken from https://blog.afoolishmanifesto.com/posts/pong/ and adjusted for this mod.

local gameNames = { "Snake", "Pong" }
local game = "None"
local gameOver = false
local screen = {}

local snake = { 
    BoardSize = { X = 10, Y = 10 },
    Render = { X = 0, Y = 0, W = 1, H = 1 },
    UpdateTime = 0.5,   -- How often the snake will move
    MoveTimer = 0,      -- How long until the snake will move
    Dir = "Up",
    Head = { X = 0, Y = 0 },
    Tail = {},
    GameStarted = false,
    ChangedDir = false,
    Food = { X = 0, Y = 0 }
}


local pongball = { 
    BoardSize = { X = 20, Y = 20 },
    Render = { X = 10.5, Y = 10, W = 1, H = 1 },
    UpdateTime = 0.1,   -- How often the ball will move
    MoveTimer = 0,      -- How long until the ball will move
    Player = "PongBall",
    GameStarted = false,
	x = 10.5,
	y = 10,
	dx = 0,
	dy = 0
}

local p1paddle = { 
    BoardSize = { X = 20, Y = 20 },
    Render = { X = 2, Y = 10, W = 1, H = 1 },
    Dir = "Up",
    Player = "PlayerLeft",
    ai = true,
    Score = 0,
	x = 2,
	y = 10
}

local p2paddle = { 
    BoardSize = { X = 20, Y = 20 },
    Render = { X = 19, Y = 10, W = 1, H = 1 },
    Dir = "Up",
    Player = "PlayerRight",
    ai = true,
    Score = 0,
	x = 19,
	y = 10
}


function Create()
    this.HasCustomReport = true
end


function Update( timePassed )
    timePassed = tonumber( timePassed )
    
    if gameOver then
        -- Do nothing
    elseif game == "Snake" then
        UpdateSnake( timePassed )
    elseif game == "Pong" then
        UpdatePong( timePassed )
    end
end



-- Begin Snake

function UpdateSnake( timePassed )
    snake.MoveTimer = snake.MoveTimer - timePassed
    
    if not snake.GameStarted then
        local t = math.floor( snake.MoveTimer )
        
        if snake.MoveTimer < 0 then
            Interface.SetComponentProperties( "StartingSnake", { Caption = "", Colour = "#41FF00" } )
            snake.GameStarted = true
            snake.MoveTimer = snake.UpdateTime
        else
            Interface.SetComponentProperties( "StartingSnake", { Caption = "arcade_snake_gamestarting", Colour = "#41FF00", Replacements = { X = tostring( t ) } } )
        end
    elseif snake.MoveTimer < 0 then
        -- Reset timer. Make sure we remove any extra time that has passed
        snake.MoveTimer = snake.UpdateTime + snake.MoveTimer
        
        -- Move now
        if SnakeCanMove() then
            MoveSnake()
            if snake.Head.X == snake.Food.X and snake.Head.Y == snake.Food.Y then
                -- We've eat some food. Spawn another body piece and a new food source
                SnakeSpawnBodyPiece()
                SnakeSpawnFood()
            end
        else
            -- Game over buddy
            Interface.SetComponentProperties( "StartingSnake", { Caption = "arcade_snake_gameover", Colour = "#41FF00" } )
            gameOver = true
        end
    end
    
    -- Remove the screen image and read it so that it's always on top. 
    -- Annoying, but bleh
    Interface.RemoveReportComponent( screen.Name )
    Interface.AddReportComponent( screen )
end


function SnakeWon()
    gameOver = true
    Interface.SetComponentProperties( "StartingSnake", { Caption = "arcade_snake_gamewon", Colour = "#41FF00" } )
end


function SnakeCanMove()
    if snake.Dir == "Up" then 
        return snake.Head.Y > 1 and not SnakeTailWillBeAt( snake.Head.X, snake.Head.Y - 1 )
    elseif snake.Dir == "Down" then 
        return snake.Head.Y < snake.BoardSize.Y and not SnakeTailWillBeAt( snake.Head.X, snake.Head.Y + 1 )
    elseif snake.Dir == "Left" then 
        return snake.Head.X > 1  and not SnakeTailWillBeAt( snake.Head.X - 1, snake.Head.Y )
    elseif snake.Dir == "Right" then 
        return snake.Head.X < snake.BoardSize.X and not SnakeTailWillBeAt( snake.Head.X + 1, snake.Head.Y )
    end
    
    return false
end


function SnakeTailWillBeAt( boardX, boardY )
    for i=(#snake.Tail) - 1,1,-1 do
        if snake.Tail[i].X == boardX and snake.Tail[i].Y == boardY then
            return true
        end
    end
    
    return false
end


function MoveSnake()
    local snakeSize = SnakeSize()
    local snakeX, snakeY
    
    -- Move all body positions to the position of the previous body position
    local num = #snake.Tail
    for i=num,2,-1 do
        snake.Tail[ i ].X = snake.Tail[ i - 1 ].X
        snake.Tail[ i ].Y = snake.Tail[ i - 1 ].Y
        
        snakeX, snakeY = SnakeRenderPos( snake.Tail[ i ].X, snake.Tail[ i ].Y )        
        Interface.SetComponentProperties( "SnakeTail"..tostring(i), { X = snakeX, Y = snakeY, W = snakeSize, H = snakeSize } )
    end
    
    -- Move the first body piece to the head piece
    if num > 0 then
        snake.Tail[ 1 ].X = snake.Head.X
        snake.Tail[ 1 ].Y = snake.Head.Y
        
        snakeX, snakeY = SnakeRenderPos( snake.Tail[ 1 ].X, snake.Tail[ 1 ].Y )
        Interface.SetComponentProperties( "SnakeTail1", { X = snakeX, Y = snakeY, W = snakeSize, H = snakeSize } )
    end
    
    -- Move the head to it's new place
    if snake.Dir == "Up" then snake.Head.Y = snake.Head.Y - 1
    elseif snake.Dir == "Down" then snake.Head.Y = snake.Head.Y + 1
    elseif snake.Dir == "Left" then snake.Head.X = snake.Head.X - 1
    elseif snake.Dir == "Right" then snake.Head.X = snake.Head.X + 1
    end
    
    -- Update the head image position
    snakeX, snakeY = SnakeRenderPos( snake.Head.X, snake.Head.Y ) 
    Interface.SetComponentProperties( "SnakeHead", { X = snakeX, Y = snakeY, W = snakeSize, H = snakeSize } )
    
    snake.ChangedDir = false
end


function SnakeSize()
    return snake.Render.W / snake.BoardSize.X
end


function SnakeRenderPos( boardX, boardY )
    local snakeSize = SnakeSize()
    local x = snake.Render.X + ( snakeSize * ( boardX - 1 ) )
    local y = snake.Render.Y + ( snakeSize * ( boardY - 1 ) )
    
    return x, y
end


function SnakeSpawnBodyPiece()
    -- Spawn the new body piece at the tail position (or head if there is no tail)
    local newTail
    local num = #snake.Tail
    if num == 0 then
        newTail = { X = snake.Head.X, Y = snake.Head.Y }
    else
        newTail = { X = snake.Tail[ num ].X, Y = snake.Tail[ num ].Y }
    end
    
    table.insert( snake.Tail, newTail )
    
    -- Add a new image to the interface
    local snakeSize = SnakeSize()
    local snakeX, snakeY = SnakeRenderPos( newTail.X, newTail.Y )
    Interface.AddReportComponent( { Type = "Image", Name = "SnakeTail" .. #snake.Tail, File = "snaketail.png", Colour = "#41FF00", X = snakeX, Y = snakeY, W = snakeSize, H = snakeSize } )
end


function SnakeSpawnFood()
    -- List of all spaces
    local spaces = {}
    for x=1,snake.BoardSize.X,1 do
        spaces[x] = {}
        for y=1,snake.BoardSize.Y,1 do
            spaces[x][y] = true
        end
    end
    
    -- Remove head and body piece locations
    spaces[ snake.Head.X ][ snake.Head.Y ] = false
    for tailI, tailPart in ipairs( snake.Tail ) do
        spaces[ tailPart.X ][ tailPart.Y ] = false
    end
    
    -- List empty spaces
    local emptySpaces = {}
    for x=1,snake.BoardSize.X,1 do
        for y=1,snake.BoardSize.Y,1 do
            if spaces[x][y] then
                table.insert( emptySpaces, { X=x, Y=y } )
            end
        end
    end
    
    local num = #emptySpaces
    if num == 0 then
        SnakeWon()
        return
    end
    
    -- Random space
    local food = emptySpaces[ math.random( num ) ]
    snake.Food.X = food.X
    snake.Food.Y = food.Y
    
    local renderX, renderY = SnakeRenderPos( food.X, food.Y )
    local renderSize = SnakeSize()
    Interface.SetComponentProperties( "SnakeFood", { X = renderX, Y = renderY, W = renderSize, H = renderSize, Angle = 0 } )
end


function ButtonPressed_Up()
    if not snake.ChangedDir and game == "Snake" then
        if snake.Dir ~= "Down" then
            snake.Dir = "Up"
            Interface.SetComponentProperties( "SnakeHead", { Angle = 0 } )
            snake.ChangedDir = true
        end
    end
end


function ButtonPressed_Down()
    if not snake.ChangedDir and  game == "Snake" then
        if snake.Dir ~= "Up" then
            snake.Dir = "Down"
            Interface.SetComponentProperties( "SnakeHead", { Angle = math.pi } )
            snake.ChangedDir = true
        end
    end
end


function ButtonPressed_Left()            
    if not snake.ChangedDir and  game == "Snake" then
        if snake.Dir ~= "Right" then
            snake.Dir = "Left"
            Interface.SetComponentProperties( "SnakeHead", { Angle = -math.pi/2 } )
            snake.ChangedDir = true
        end
    end
end


function ButtonPressed_Right()
    if not snake.ChangedDir and  game == "Snake" then
        if snake.Dir ~= "Left" then
            snake.Dir = "Right"
            Interface.SetComponentProperties( "SnakeHead", { Angle = math.pi/2 } )
            snake.ChangedDir = true
        end
    end
end


function ButtonPressed_SnakeCentre()
    if gameOver then
		gameOver = false
		ResetSnakeGameState( snake.Render.W, snake.Render.H )
	else
		gameOver = true
		snake.GameStarted = false
		ResetSnakeGameState( snake.Render.W, snake.Render.H )
		Interface.SetComponentProperties( "StartingSnake", { Caption = "arcade_snake_pressbutton", Colour = "#41FF00" } )
    end
end


-- End Snake

-- Begin Pong


function ButtonPressed_P1Up()
	p1paddle.Dir = "Up"
	p1paddle.ai = false
end


function ButtonPressed_P1Down()
	p1paddle.Dir = "Down"
	p1paddle.ai = false
end


function ButtonPressed_P2Up()
	p2paddle.Dir = "Up"
	p2paddle.ai = false
end


function ButtonPressed_P2Down()
	p2paddle.Dir = "Down"
	p2paddle.ai = false
end


function ButtonPressed_P1Ai()
	p1paddle.ai = true
end


function ButtonPressed_PongCentre()
    if gameOver then
		gameOver = false
		ResetPongGameState( pongball.Render.W, pongball.Render.H )
	else
		gameOver = true
		pongball.GameStarted = false
		ResetPongGameState( pongball.Render.W, pongball.Render.H )
		Interface.SetComponentProperties( "StartingPong", { Caption = "arcade_pong_pressbutton", Colour = "#FFB000" } )
    end
end


function ButtonPressed_P2Ai()
	p2paddle.ai = true
end


function UpdatePong( timePassed )
    pongball.MoveTimer = pongball.MoveTimer - timePassed
    
    if not pongball.GameStarted then
        local t = math.floor( pongball.MoveTimer )
        
        if pongball.MoveTimer < 0 then
            Interface.SetComponentProperties( "StartingPong", { Caption = "" } )
            pongball.GameStarted = true
            pongball.MoveTimer = pongball.UpdateTime
        else
            Interface.SetComponentProperties( "StartingPong", { Caption = "arcade_pong_gamestarting", Colour = "#FFB000", Replacements = { X = tostring( t ) } } )
        end
    elseif pongball.MoveTimer < 0 then
        -- Reset timer. Make sure we remove any extra time that has passed
        pongball.MoveTimer = pongball.UpdateTime + pongball.MoveTimer
        
		update_player(p1paddle, pongball)
		update_player(p2paddle, pongball)
		update_ball(pongball)
		intersection(p1paddle, p2paddle, pongball)
		
        if p1paddle.Score == 10 or p2paddle.Score == 10 then
            -- Game over buddy
            Interface.SetComponentProperties( "StartingPong", { Caption = "arcade_pong_gameover", Colour = "#FFB000" } )
            gameOver = true
		else
			Interface.SetComponentProperties( "StartingPong", { Caption = "arcade_pong_scoreboard", Colour = "#FFB000", Replacements = { X = p1paddle.Score, Y = p2paddle.Score } } )
        end
    end
    
    -- Remove the screen image and read it so that it's always on top. 
    -- Annoying, but bleh
    Interface.RemoveReportComponent( screen.Name )
    Interface.AddReportComponent( screen )
end


function do_ai(p, b)
	if b.y < p.y and p.y > 1 then
		p.y = p.y - 0.5
	elseif b.y > p.y and p.y < 19 then
		p.y = p.y + 0.5
	end
end


function update_player(p, b)
	if not p.ai then
		if p.Dir == "Up" and p.y > 1 then p.y = p.y - 0.5 end
		if p.Dir == "Down" and p.y < 19 then p.y = p.y + 0.5 end
	else
		do_ai(p, b)
	end
	p.ChangedDir = false
    local paddleSize = PaddleSize(p)
	local paddleX, paddleY = PaddleRenderPos( p, p.x, p.y )
	Interface.SetComponentProperties( p.Player.."PaddleHead", { X = paddleX, Y = paddleY, W = paddleSize, H = paddleSize*2 } )
end


function update_ball(b)
	b.x = b.x + b.dx
	b.y = b.y + b.dy
    local pongSize = PaddleSize(b)
	local pongX, pongY = PaddleRenderPos( b, b.x, b.y )
	Interface.SetComponentProperties( "PongBall", { X = pongX, Y = pongY, W = pongSize, H = pongSize } )
end

function reset_ball(b,s)
	b.x  = s.x
	b.y  = s.y
	if s.Player == "PlayerLeft" then
		b.dx = 0.25
	elseif s.Player == "PlayerRight" then
		b.dx = -0.25
	end
	b.dy = math.random()
end

function intersection(l, r, b)
	calc_angle = false
	p = {}
	s = {}
	
	print("b.x: "..b.x.." b.y: "..b.y.."   l.x: "..l.x.." l.y: "..l.y.."   r.x: "..r.x.." r.y: "..r.y)
	if b.x >= l.x and b.x <= l.x+0.5 and b.y >= l.y-1 and b.y <= l.y + 2 then
		print("1d")
		b.x = l.x + 0.5
		b.dx = -b.dx
		calc_angle = true
		p = l
	elseif b.x >= r.x-0.5 and b.x <= r.x and b.y >= r.y-1 and b.y <= r.y + 2 then
		print("1e")
		b.x = r.x - 0.5
		b.dx = -b.dx
		calc_angle = true
		p = r
	elseif b.x < 1 then
		print("1a")
		r.Score = r.Score + 1
		print("SCORE:  "..l.Score.." - "..r.Score)
		s = r
		reset_ball(b,s)
	elseif b.x > 20 then
		print("1b")
		l.Score = l.Score + 1
		print("SCORE:  "..l.Score.." - "..r.Score)
		s = l
		reset_ball(b,s)
	elseif b.y < 1 or b.y > 20 then
		print("1c")
		b.dy = -b.dy
		calc_angle = true
		p.y = b.dy
	end
	if calc_angle then
		print("1f")
		if math.random() > 0.9 then
			if b.dx >= 0 then
				b.dx = b.dx + 0.05
			else
				b.dx = b.dx - 0.05
			end
		end
		rl = (b.y - p.y) / 2
		rl = rl / 2 + 0.25
		angle = math.sin(rl)
		b.dy = angle
	end
end


function PaddleSize(thePaddle)
    return (thePaddle.Render.W / thePaddle.BoardSize.X)
end


function PaddleRenderPos(thePaddle, boardX, boardY )
    local paddleSize = PaddleSize(thePaddle)
    local x = thePaddle.Render.X + ( paddleSize * ( boardX - 1 ) )
    local y = thePaddle.Render.Y + ( paddleSize * ( boardY - 1 ) )
    return x, y
end


-- Custom Report Window

function SetTabNames()
    -- Game names make our tabs
    Interface.SetReportTabs( 
        "arcade_game_snake"
        ,"arcade_game_pong" 
    )
end


function PopulateReport( tabIndex )
    game = gameNames[ tonumber( tabIndex ) ]
    local w, h = Interface.GetReportWindowSize()
    
    SetupCommonComponents( w, h )
    
    if game == "Snake" then
        SetupSnake( w, h )
    elseif game == "Pong" then
        SetupPong( w, h )
    end
    
end


function SetupSnake( w, h )
    -- Add some extra components:
    -- Snake head
    Interface.AddReportComponent( { Type = "Image", Name = "SnakeHead", File = "snakehead.png", Colour = "#41FF00" } )
    Interface.AddReportComponent( { Type = "Image", Name = "SnakeFood", File = "snaketail.png", Colour = "Red" } )
    
    local minSize = math.min( w, h )
    local border = minSize / 10
    local borderX = border
    local borderY = border
    local borderX2 = minSize - border
    local borderY2 = minSize - border
    
    -- Starting caption
    Interface.AddReportComponent( { Type = "Caption", Name = "StartingSnake", Caption = "arcade_snake_gamestarting", Centred = true, 
        X = 0, Y = borderY2, W = w, H = border, Colour = "#41FF00", Replacements = { X = "5" } } )
    
    -- Render information
    snake.Render.X = borderX
    snake.Render.Y = borderY
    snake.Render.W = borderX2 - borderX
    snake.Render.H = borderY2 - borderY
    
    ResetSnakeGameState()
end


function ResetSnakeGameState()
    -- Remove old tail images
    for i=1,#snake.Tail,1 do
        Interface.RemoveReportComponent( "SnakeTail" .. tostring( i ) )
    end
    
    -- Set up our game state
    snake.MoveTimer = 5
    
    snake.Head.X = math.floor( snake.BoardSize.X / 2 )
    snake.Head.Y = math.floor( snake.BoardSize.Y / 2 )
    
    snake.Tail = {}
    
    snake.Dir = "Up"
    snake.GameStarted = false
    snake.ChangedDir = false
    
    SnakeSpawnFood()
    
    local snakeSize = SnakeSize()
    local snakeX, snakeY = SnakeRenderPos( snake.Head.X, snake.Head.Y )
    Interface.SetComponentProperties( "SnakeHead", { X = snakeX, Y = snakeY, W = snakeSize, H = snakeSize, Angle = 0 } )
    
    Interface.SetComponentProperties( "StartingSnake", { Caption = "arcade_snake_gamestarting", Colour = "#41FF00", Replacements = { X = tostring(math.floor(snake.MoveTimer)) } } )
end


function SetupPong( w, h )
    -- Add some extra components:
    -- Pong ball and left/right bat
    Interface.AddReportComponent( { Type = "Image", Name = "PongBall", File = "pongball.png", Colour = "#FFB000" } )
    Interface.AddReportComponent( { Type = "Image", Name = p1paddle.Player.."PaddleHead", File = "pongpaddle.png", Colour = "#FFB000" } )
    Interface.AddReportComponent( { Type = "Image", Name = p2paddle.Player.."PaddleHead", File = "pongpaddle.png", Colour = "#FFB000" } )
    
    local minSize = math.min( w, h )
    local border = minSize / 10
    local borderX = border
    local borderY = border
    local borderX2 = minSize - border
    local borderY2 = minSize - border
    
    -- Starting caption
    Interface.AddReportComponent( { Type = "Caption", Name = "StartingPong", Caption = "arcade_pong_gamestarting", Centred = true, 
        X = 0, Y = borderY2, W = w, H = border, Colour = "#FFB000", Replacements = { X = "5" } } )
    
    -- Render information
    pongball.Render.X = borderX
    pongball.Render.Y = borderY
    pongball.Render.W = borderX2 - borderX
    pongball.Render.H = borderY2 - borderY
    
    p1paddle.Render.X = borderX
    p1paddle.Render.Y = borderY
    p1paddle.Render.W = borderX2 - borderX
    p1paddle.Render.H = borderY2 - borderY
	
    p2paddle.Render.X = borderX
    p2paddle.Render.Y = borderY
    p2paddle.Render.W = borderX2 - borderX
    p2paddle.Render.H = borderY2 - borderY
    ResetPongGameState()
end


function ResetPongGameState()

    -- Set up our game state
    pongball.MoveTimer = 5
    pongball.GameStarted = false
	
	if math.random() > 0.5 then
		pongball.dx = 0.25
	else
		pongball.dx = -0.25
	end
	if math.random() > 0.5 then
		pongball.dy = -(math.random())
	else
		pongball.dy = math.random()
	end
	
    pongball.x = 10.5
    pongball.y = 10.5
    
    p1paddle.x = 2
    p1paddle.y = math.floor( p1paddle.BoardSize.Y / 2 )
	
    p2paddle.x = 19
    p2paddle.y = math.floor( p2paddle.BoardSize.Y / 2 )
	
    p1paddle.Dir = "Up"
	p1paddle.Score = 0
    p1paddle.ChangedDir = false
	
    p2paddle.Dir = "Up"
	p2paddle.Score = 0
    p2paddle.ChangedDir = false
    
    local pongSize = PaddleSize(pongball)
    local pongX, pongY = PaddleRenderPos( pongball, pongball.x, pongball.y )
    Interface.SetComponentProperties( "PongBall", { X = pongX, Y = pongY, W = pongSize, H = pongSize } )
	
    local paddle1Size = PaddleSize(p1paddle)
    local paddle1X, paddle1Y = PaddleRenderPos( p1paddle, p1paddle.x, p1paddle.y )
    Interface.SetComponentProperties( p1paddle.Player.."PaddleHead", { X = paddle1X, Y = paddle1Y, W = paddle1Size, H = paddle1Size*2 } )
    
    local paddle2Size = PaddleSize(p2paddle)
    local paddle2X, paddle2Y = PaddleRenderPos( p2paddle, p2paddle.x, p2paddle.y )
    Interface.SetComponentProperties( p2paddle.Player.."PaddleHead", { X = paddle2X, Y = paddle2Y, W = paddle2Size, H = paddle2Size*2 } )
    
    Interface.SetComponentProperties( "StartingPong", { Caption = "arcade_pong_gamestarting", Colour = "#FFB000", Replacements = { X = tostring(math.floor(pongball.MoveTimer)) } } )
end


function SetupCommonComponents( w, h )
    local size = math.min( w, h )
    local y = 0
    
	
    if game == "Snake" then
	
		background = {
			Type = "Image",
			Name = "Background",
			X = (w / 2 ) - ( size / 2 ),
			Y = y,
			W = size,
			H = size,
			Colour = "White",
			File = "backgroundsnake.png"
		}
		
		screen = {
			Type = "Image",
			Name = "ScreenSnake",
			X = (w / 2 ) - ( size / 2 ),
			Y = y,
			W = size,
			H = size,
			Colour = "White",
			File = "screensnake.png"
		}
		
    elseif game == "Pong" then
	
		background = {
			Type = "Image",
			Name = "Background",
			X = (w / 2 ) - ( size / 2 ),
			Y = y,
			W = size,
			H = size,
			Colour = "White",
			File = "backgroundpong.png"
		}
		
		screen = {
			Type = "Image",
			Name = "ScreenPong",
			X = (w / 2 ) - ( size / 2 ),
			Y = y,
			W = size,
			H = size,
			Colour = "White",
			File = "screenpong.png"
		}
    end
    
    -- Title
    Interface.AddReportComponent( { Type = "Title", Caption = "arcade_title" } )
    
    -- Background and screen
    Interface.AddReportComponent( background )
    Interface.AddReportComponent( screen )    
    
    y = y + background.H
    
    if game == "Snake" then
		-- Buttons for Snake. Up, down, left, right, and centre
		local buttonSize = w / 16
		local buttonGap = buttonSize / 4
		
		local up = { Type = "Button", Name = "Up", X = (w / 2 ) - ( buttonSize / 2 ), Y = y + buttonGap, W = buttonSize, H = buttonSize, Caption = "" }
		local centre =  { Type = "Button", Name = "SnakeCentre", X = up.X, Y = up.Y + buttonSize + buttonGap, W = buttonSize, H = buttonSize, Caption = "" }
		local down = { Type = "Button", Name = "Down", X = up.X, Y = centre.Y + buttonSize + buttonGap, W = buttonSize, H = buttonSize, Caption = "" }
		local left = { Type = "Button", Name = "Left", X = centre.X - buttonSize - buttonGap, Y = centre.Y, W = buttonSize, H = buttonSize, Caption = "" }
		local right = { Type = "Button", Name = "Right", X = centre.X + buttonSize + buttonGap, Y = centre.Y, W = buttonSize, H = buttonSize, Caption = "" }
		
		Interface.AddReportComponent( up )
		Interface.AddReportComponent( centre )
		Interface.AddReportComponent( down )
		Interface.AddReportComponent( left )
		Interface.AddReportComponent( right )
		
		-- Cover the buttons with some nice images
		up.Name = "ImageUp"
		down.Name = "ImageDown"
		left.Name = "ImageLeft"
		right.Name = "ImageRight"
		centre.Name = "ImageCentre"
		
		up.Type = "Image"
		down.Type = "Image"
		left.Type = "Image"
		right.Type = "Image"
		centre.Type = "Image"
		
		up.File = "arrowbutton.png"
		down.File = "arrowbutton.png"
		left.File = "arrowbutton.png"
		right.File = "arrowbutton.png"
		centre.File = "circlebutton.png"
		
		up.Colour = "White"
		down.Colour = "White"
		left.Colour = "White"
		right.Colour = "White"
		centre.Colour = "Red"
		
		down.Angle = math.pi
		left.Angle = -math.pi/2
		right.Angle = math.pi/2
		
		Interface.AddReportComponent( up )
		Interface.AddReportComponent( centre )
		Interface.AddReportComponent( down )
		Interface.AddReportComponent( left )
		Interface.AddReportComponent( right )
    elseif game == "Pong" then
		local buttonSize = w / 16
		local buttonGap = buttonSize / 4
		
		local P1up = { Type = "Button", Name = "P1Up", X = buttonSize*2, Y = y + buttonGap, W = buttonSize, H = buttonSize, Caption = "" }
		local P1down = { Type = "Button", Name = "P1Down", X = buttonSize*2, Y = P1up.Y + buttonSize + buttonGap, W = buttonSize, H = buttonSize, Caption = "" }
		local P1ai = { Type = "Button", Name = "P1Ai", X = (w / 6 ) + buttonSize, Y = P1up.Y + ( buttonSize / 2 ), W = buttonSize, H = buttonSize, Caption = "" }
		local centre = { Type = "Button", Name = "PongCentre", X = (w / 2 ) - ( buttonSize / 2 ), Y = P1up.Y + ( buttonSize / 2 ), W = buttonSize, H = buttonSize, Caption = "" }
		local P2ai = { Type = "Button", Name = "P2Ai", X = (w / 3 ) + (w / 3 ) + (buttonSize / 2), Y = P1up.Y + ( buttonSize / 2 ), W = buttonSize, H = buttonSize, Caption = "" }
		local P2up = { Type = "Button", Name = "P2Up", X = w - buttonSize*3, Y = y + buttonGap, W = buttonSize, H = buttonSize, Caption = "" }
		local P2down = { Type = "Button", Name = "P2Down", X = w - buttonSize*3, Y = P2up.Y + buttonSize + buttonGap, W = buttonSize, H = buttonSize, Caption = "" }
		
		Interface.AddReportComponent( P1up )
		Interface.AddReportComponent( P1ai )
		Interface.AddReportComponent( centre )
		Interface.AddReportComponent( P2ai )
		Interface.AddReportComponent( P1down )
		Interface.AddReportComponent( P2up )
		Interface.AddReportComponent( P2down )
		
		-- Cover the buttons with some nice images
		P1up.Name = "ImageP1Up"
		P1down.Name = "ImageP1Down"
		P1ai.Name = "ImageP1ai"
		centre.Name = "ImageCentre"
		P2ai.Name = "ImageP2ai"
		P2up.Name = "ImageP2Up"
		P2down.Name = "ImageP2Down"
		
		P1up.Type = "Image"
		P1down.Type = "Image"
		P1ai.Type = "Image"
		centre.Type = "Image"
		P2ai.Type = "Image"
		P2up.Type = "Image"
		P2down.Type = "Image"
		
		P1up.File = "arrowbutton.png"
		P1down.File = "arrowbutton.png"
		P1ai.File = "aibutton.png"
		centre.File = "circlebutton.png"
		P2ai.File = "aibutton.png"
		P2up.File = "arrowbutton.png"
		P2down.File = "arrowbutton.png"
		
		P1up.Colour = "White"
		P1down.Colour = "White"
		P1ai.Colour = "White"
		centre.Colour = "Red"
		P2ai.Colour = "White"
		P2up.Colour = "White"
		P2down.Colour = "White"
		
		P1down.Angle = math.pi
		P2down.Angle = math.pi
		
		Interface.AddReportComponent( P1up )
		Interface.AddReportComponent( P1ai )
		Interface.AddReportComponent( centre )
		Interface.AddReportComponent( P2ai )
		Interface.AddReportComponent( P1down )
		Interface.AddReportComponent( P2up )
		Interface.AddReportComponent( P2down )
		
    end
end

